"""
Created on Sun Dec 04 09:27:00 2022

@author: lpaone
"""
import json
import os
import unittest
from typing import Union

import pycivil.EXAStructural.templateRCRect as RCRect
from pycivil.EXAGeometry.clouds import PointCloud2d
from pycivil.EXAGeometry.geometry import Point2d
from pycivil.EXAGeometry.shapes import ShapeArea, ShapeRect
from pycivil.EXAStructural.codes import Code
from pycivil.EXAStructural.materials import Concrete, ConcreteSteel
from pycivil.EXAStructural.sections import ConcreteSection, StructSectionItem


def rectSolverDomain_NM(
    dimW,
    dimH,
    steelAreaTop,
    steelAreaBottom,
    distFromTop,
    distFromBot,
    Ned,
    Med,
    homogenization=15.0,
):

    # Setting new instance of section with
    section = RCRect.RCTemplRectEC2(1, "First Section")

    # Setting dimension concrete
    section.setDimH(dimH)
    section.setDimW(dimW)

    # Setting rebar
    section.addSteelArea("MB", distFromBot, steelAreaBottom)
    section.addSteelArea("MT", distFromTop, steelAreaTop)

    # Setting materials
    section.setMaterials(homogenization=homogenization)

    # Solver with interaction domain
    # nbPoints need to be greater than 100 to avoid wrong results
    pointCloud, bb = section.interactionDomainBuild2d(
        nbPoints=200, SLS=True, bounding=True
    )

    brox = bb[1] - bb[0]
    broy = bb[3] - bb[2]

    (contained, pintersect, intfactor, pindex) = pointCloud.contains(
        -Ned * 1e3, Med * 1e6, rayFromCenter=True, ro=(brox, broy)
    )

    Point2d(-Ned * 1e3, Med * 1e6)
    Point2d(0, 0)

    idx = pindex[0]
    dFromLeft = pindex[1]
    dFromLeftToRight = pindex[3]

    assert isinstance(pointCloud, PointCloud2d)

    sigma_c_left = pointCloud.getMetaAt(idx)[3] / intfactor
    sigma_s_left = pointCloud.getMetaAt(idx)[4] / intfactor
    xi_left = pointCloud.getMetaAt(idx)[5]

    sigma_c_right = pointCloud.getMetaAt(idx + 1)[3] / intfactor
    sigma_s_right = pointCloud.getMetaAt(idx + 1)[4] / intfactor
    xi_right = pointCloud.getMetaAt(idx + 1)[5]

    if 0 != dFromLeftToRight:
        sigma_c_domain = sigma_c_left + dFromLeft / dFromLeftToRight * (
            sigma_c_right - sigma_c_left
        )
        sigma_s_domain = sigma_s_left + dFromLeft / dFromLeftToRight * (
            sigma_s_right - sigma_s_left
        )
        xi_domain = xi_left + dFromLeft / dFromLeftToRight * (xi_right - xi_left)
    else:
        sigma_c_domain = sigma_c_left
        sigma_s_domain = sigma_s_left
        xi_domain = xi_left

    return {"sigmac": sigma_c_domain, "sigmas": sigma_s_domain, "xi": xi_domain}


def rectSolverClassic_NM(
    dimW,
    dimH,
    steelAreaTop,
    steelAreaBottom,
    distFromTop,
    distFromBot,
    Ned,
    Med,
    homogenization=15.0,
):
    # Setting new instance of section with
    section = RCRect.RCTemplRectEC2(1, "First Section")

    # Setting dimension concrete
    section.setDimH(dimH)
    section.setDimW(dimW)
    # Setting rebar
    section.addSteelArea("MB", distFromBot, steelAreaBottom)
    section.addSteelArea("MT", distFromTop, steelAreaTop)

    # Setting materials
    section.setMaterials(homogenization=homogenization)

    # Solver with roots
    #
    sol = section.solverSLS_NM(N=Ned * 1e3, M=Med * 1e6)

    return {"sigmac": sol[0], "sigmas": sol[1], "xi": sol[2]}


def relativeError(valCal: float, valRef: Union[float, None]):
    if valRef is None:
        return 0.0
    else:
        return abs((valRef - valCal) / valRef)


class Test(unittest.TestCase):
    def __init__(self, *args, **kwargs):

        super().__init__(*args, **kwargs)

        concrete = Concrete()
        steel = ConcreteSteel()
        self.sec = ConcreteSection()
        self.sec.setStructConcrItem(
            StructSectionItem(
                shape=ShapeRect(ids=1, width=300, height=600), material=concrete
            )
        )
        self.sec.addSteelArea(
            StructSectionItem(
                shape=ShapeArea(area=314, ids=10, xo=-100.0, yo=-250.0), material=steel
            )
        )
        self.sec.addSteelArea(
            StructSectionItem(
                shape=ShapeArea(area=314, ids=11, xo=+100.0, yo=-250.0), material=steel
            )
        )
        self.sec.addSteelArea(
            StructSectionItem(
                shape=ShapeArea(area=200, ids=12, xo=+100.0, yo=+250.0), material=steel
            )
        )
        self.sec.addSteelArea(
            StructSectionItem(
                shape=ShapeArea(area=200, ids=13, xo=-100.0, yo=+250.0), material=steel
            )
        )

    def test_110_Rectangular_Shape_getArea(self):
        self.assertEqual(
            300 * 600, self.sec.getStructConcretelItem().getShape().getArea()
        )

    def test_112_Rectangular_Shape_getOrigin(self):
        self.assertEqual(
            0.0, self.sec.getStructConcretelItem().getShape().getOrigin().x
        )
        self.assertEqual(
            0.0, self.sec.getStructConcretelItem().getShape().getOrigin().y
        )

    def test_120_Rectangular_Shape_translate_concreteOrigin(self):
        shape = self.sec.getStructConcretelItem().getShape()
        shape.translate(Point2d(0.0, 0.0), Point2d(150, 0))
        self.assertEqual(
            150.0, self.sec.getStructConcretelItem().getShape().getOrigin().x
        )

    def test_121_Rectangular_Shape_translate_concreteItem(self):
        shape = self.sec.getStructConcretelItem().getShape()
        shape.translate(Point2d(0.0, 0.0), Point2d(0, 300))
        self.assertEqual(
            300.0, self.sec.getStructConcretelItem().getShape().getOrigin().y
        )

    def test_122_Rectangular_Shape_translateConcreteItem(self):
        # reset position
        self.sec.getStructConcretelItem().getShape().setOrigin(Point2d(0.0, 0.0))

        self.sec.translateConcreteItem(Point2d(0.0, 0.0), Point2d(150, 300))
        self.assertEqual(
            300.0, self.sec.getStructConcretelItem().getShape().getOrigin().y
        )
        self.assertEqual(
            150.0, self.sec.getStructConcretelItem().getShape().getOrigin().x
        )

    def test_123_Rectangular_Shape_translateSteelItems(self):
        # reset position
        self.sec.getStructConcretelItem().getShape().setOrigin(Point2d(0.0, 0.0))

        # translate steel rebars
        self.sec.translateSteelItems(Point2d(0.0, 0.0), Point2d(5.0, 5.0))

        self.assertEqual(-95.0, self.sec.getSteelRebar()[0].getOrigin().x)
        self.assertEqual(-245.0, self.sec.getSteelRebar()[0].getOrigin().y)
        self.assertEqual(105.0, self.sec.getSteelRebar()[1].getOrigin().x)
        self.assertEqual(-245.0, self.sec.getSteelRebar()[1].getOrigin().y)
        self.assertEqual(105.0, self.sec.getSteelRebar()[2].getOrigin().x)
        self.assertEqual(255.0, self.sec.getSteelRebar()[2].getOrigin().y)
        self.assertEqual(-95.0, self.sec.getSteelRebar()[3].getOrigin().x)
        self.assertEqual(255.0, self.sec.getSteelRebar()[3].getOrigin().y)

    def test_200_Rectangular_Domain(self):
        """
        ....dimension: 300x500-code: EC2-concrete: C32/40-steel: B450C
        """
        # Setting working code
        code_EC2 = Code("EC2")

        # Setting concrete material
        cls_material = Concrete(descr="EC2_C32/40")
        cls_material.setByCode(code_EC2, "C32/40")

        # Setting concrete shape
        rect_shape = ShapeRect(300, 500)

        rectangularSection = StructSectionItem(rect_shape, cls_material)

        # Setting steel material
        steel_material = ConcreteSteel(descr="EC2_450C")
        steel_material.setByCode(code_EC2, "B450C")

        # Retriving special points
        MB = rectangularSection.getShape().getShapePoint("MB")
        MT = rectangularSection.getShape().getShapePoint("MT")

        # Setting bottom steel shape
        area_shape_1 = ShapeArea(600)
        area_shape_1.setOrigin(MB + Point2d(0, 38))
        steel_1 = StructSectionItem(area_shape_1, steel_material)

        # Setting top steel shape
        area_shape_2 = ShapeArea(600)
        area_shape_2.setOrigin(MT + Point2d(0, -40))
        steel_2 = StructSectionItem(area_shape_2, steel_material)

        # Forming section
        myfirstsection = ConcreteSection(1, "300x500 EC2")
        myfirstsection.setStructConcrItem(rectangularSection)
        myfirstsection.setStructSteelItems([steel_1, steel_2])

        # Stetting material for whole secction
        myfirstsection.setConcreteMaterial(cls_material)
        myfirstsection.setSteelMaterial(steel_material)

        (
            NxMz,
            fields,
            NxMzBoundingRect,
        ) = myfirstsection.build2dInteractionCompleteDomain()

        domain = {"points": NxMz, "fields": fields, "bounding": NxMzBoundingRect}

        # NOTE: Decomment this only for create benchmark file
        #
        # with open(os.path.join(os.path.dirname(__file__),'test_200_Rectangular_Domain.json'), 'w') as outfile:
        #     json.dump(domain,outfile,indent=4)

        fileName = os.path.join(
            os.path.dirname(__file__), "test_200_Rectangular_Domain.json"
        )
        with open(fileName) as jsonFile:
            jsonObject = json.load(jsonFile)
            jsonFile.close()

        for i, p in enumerate(jsonObject["points"]):
            self.assertAlmostEqual(domain["points"][i][0], p[0])

        for i, p in enumerate(jsonObject["fields"]):
            self.assertAlmostEqual(domain["fields"][i], p)

        for i, p in enumerate(jsonObject["bounding"]):
            self.assertAlmostEqual(domain["bounding"][i], p)

    def autoTestFun(self, tollReg, sigmacReg, sigmasReg, xiReg, sol, title):
        print("\n")
        print("-" * len(title))
        print(title)
        print("-" * len(title))

        if sigmacReg is not None:
            self.assertLessEqual(
                relativeError(sol["sigmac"], sigmacReg),
                tollReg,
                "sigmac \n from cal {:.9f} \n from ref {:.9f} ... failed".format(
                    sol["sigmac"], sigmacReg
                ),
            )
            print(
                "sigmac \n from cal {:.9f} \n from ref {:.9f} ... passed".format(
                    sol["sigmac"], sigmacReg
                )
            )

        if sigmasReg is not None:
            self.assertLessEqual(
                relativeError(sol["sigmas"], sigmasReg),
                tollReg,
                "sigmas \n from cal {:.9f} \n from ref {:.9f} ... failed".format(
                    sol["sigmas"], sigmasReg
                ),
            )
            print(
                "sigmas \n from cal {:.9f} \n from ref {:.9f} ... passed".format(
                    sol["sigmas"], sigmasReg
                )
            )

        if xiReg is not None:
            self.assertLessEqual(
                relativeError(sol["xi"], xiReg),
                tollReg,
                "xi \n from cal {:.9f} \n from ref {:.9f} ... failed".format(
                    sol["xi"], xiReg
                ),
            )
            print(
                "xi \n from cal {:.9f} \n from ref {:.9f} ... passed".format(
                    sol["xi"], xiReg
                )
            )

    def test_301_solver_NM(self):

        sectionVal = {
            "dimW": 1000.0,
            "dimH": 1300.0,
            "steelAreaTop": 1571.0,
            "steelAreaBottom": 3142.0,
            "distFromTop": 50.0,
            "distFromBot": 50.0,
            "Ned": -964.0,
            "Med": +1081.0,
        }

        title = "Classic Solution not regression"
        solClassic = rectSolverClassic_NM(**sectionVal)
        self.autoTestFun(
            1e-7, -6.165762118, 168.627969091, 442.748616437, solClassic, title
        )

        sectionVal["steelAreaTop"] = 3142.0
        sectionVal["steelAreaBottom"] = 1571.0
        sectionVal["Med"] = -1081.0

        title = "Classic Solution Mirrored not regression"
        solClassic = rectSolverClassic_NM(**sectionVal)
        self.autoTestFun(
            1e-7, -6.165762118, 168.627969091, 1300 - 442.748616437, solClassic, title
        )

        title = "Classic Solution with reference sws_sheet_1.pdf nb.1 (+)"
        self.autoTestFun(1e-3, -6.17, 168.70, None, solClassic, title)

        title = "Domain Solution not regression"
        solDomain = rectSolverDomain_NM(**sectionVal)
        self.autoTestFun(
            1e-7, -6.165718127, 168.630377249, 442.754601582, solDomain, title
        )

    def test_302_solver_NM(self):

        sectionVal = {
            "dimW": 1000.0,
            "dimH": 2300.0,
            "steelAreaTop": 1901.0,
            "steelAreaBottom": 3801.0,
            "distFromTop": 50.0,
            "distFromBot": 50.0,
            "Ned": -2123.6,
            "Med": +941.5,
        }

        title = "Classic Solution not regression"
        solClassic = rectSolverClassic_NM(**sectionVal)
        self.autoTestFun(
            1e-7, -1.904332499, 0.940037791, 2178.314420458, solClassic, title
        )

        sectionVal["steelAreaTop"] = 3801.0
        sectionVal["steelAreaBottom"] = 1901.0
        sectionVal["Med"] = -941.5

        title = "Classic Solution Mirrored not regression"
        solClassic = rectSolverClassic_NM(**sectionVal)
        self.autoTestFun(
            1e-7, -1.904332499, 0.940037791, 121.685579542, solClassic, title
        )

        title = "Classic Solution with reference sws_sheet_1.pdf nb.2 (+)"
        self.autoTestFun(1e-2, -1.90, 0.94, None, solClassic, title)

        title = "Domain Solution not regression"
        solDomain = rectSolverDomain_NM(**sectionVal)
        self.autoTestFun(
            1e-7, -1.904647142, 0.950300863, 2177.986768409, solDomain, title
        )

    def test_303_solver_NM(self):

        sectionVal = {
            "dimW": 1000.0,
            "dimH": 2200.0,
            "steelAreaTop": 1901.0,
            "steelAreaBottom": 3801.0,
            "distFromTop": 50.0,
            "distFromBot": 50.0,
            "Ned": -2123.8,
            "Med": +1695.4,
        }

        title = "Classic Solution not regression"
        solClassic = rectSolverClassic_NM(**sectionVal)
        self.autoTestFun(
            1e-8, -3.389217826, 34.788136630, 1276.501988257, solClassic, title
        )

        sectionVal["steelAreaTop"] = 3801.0
        sectionVal["steelAreaBottom"] = 1901.0
        sectionVal["Med"] = -1695.4

        title = "Classic Solution Mirrored not regression"
        solClassic = rectSolverClassic_NM(**sectionVal)
        self.autoTestFun(
            1e-8, -3.389217826, 34.788136630, 923.498011743, solClassic, title
        )

        title = "Classic Solution with reference sws_sheet_1.pdf nb.3 (+)"
        self.autoTestFun(1e-2, -3.39, 34.78, None, solClassic, title)

        title = "Domain Solution not regression"
        solDomain = rectSolverDomain_NM(**sectionVal)
        self.autoTestFun(
            1e-9, -3.389314680, 34.797034357, 1276.484106795, solDomain, title
        )


if __name__ == "__main__":
    unittest.main()



from pathlib import Path
import pytest

from pycivil.EXAStructural.codes import Code
from pycivil.EXAUtils.latexReportMakers import CodesFB
from pycivil.EXAUtils.report import (
    Reporter,
    ReportTemplateEnum,
    getTemplatesPath,
)

@pytest.mark.needLatex
def test_report_latex_codes(tmp_path: Path):

    c = Code()
    c.setCodeStr("EC2:ITA")
    print(c.tabKeys)

    c1 = Code("NTC2008")
    c2 = Code("EC2:ITA")
    c3 = Code("CIRC2008")
    c4 = Code("CIRC2018")
    c5 = Code("EC2")

    latexTemplatePath = getTemplatesPath()
    codes = CodesFB(latexTemplatePath, [c1, c2, c3])
    codes.appendUniqueCode(c4)
    codes.appendUniqueCode(c3)
    codes.appendUniqueCode(c5)

    codes.setOptionNameList("Laws used")
    codes.setOptionEnvironnment(True)
    codes.setOptionSection()

    reporter = Reporter(latexTemplatePath)
    reporter.linkFragments(
        template=ReportTemplateEnum.TEX_ENG_CAL,
        fragments=[codes.buildFragment()],
    )
    reporter.compileDocument(path=str(tmp_path))
    assert tmp_path.joinpath("report.pdf").exists()
    assert tmp_path.joinpath("report.tex").exists()

@pytest.mark.needLatex
def test_report_latex_codes_with_str(tmp_path: Path):
    codes = CodesFB(getTemplatesPath())

    codes.appendUniqueCode(code_str="NTC2008")
    codes.appendUniqueCode(code_str="EC2:ITA")
    codes.appendUniqueCode(code_str="CIRC2008")
    codes.appendUniqueCode(code_str="CIRC2018")
    codes.appendUniqueCode(code_str="EC2")

    codes.setOptionNameList("Laws used")
    codes.setOptionEnvironnment(True)
    codes.setOptionSection()

    reporter = Reporter(getTemplatesPath())

    reporter.linkFragments(
        template=ReportTemplateEnum.TEX_ENG_CAL,
        builder=codes,
    )

    reporter.compileDocument(path=str(tmp_path))
    assert tmp_path.joinpath("report.pdf").exists()
    assert tmp_path.joinpath("report.tex").exists()



from enum import Enum
from typing import Optional
from uuid import UUID

from pydantic import BaseModel

class UserRole(str, Enum):
    USER = "user"
    ADMIN = "admin"
    SUPER = "super"
    NONE = "none"

class User(BaseModel):
    userId: str = ""
    uuid: UUID = UUID("{00000000-0000-0000-0000-000000000000}")
    active: bool = True
    usr: str = ""
    psw: str = ""
    mailLogin: str = ""
    name: str = ""
    surname: str = ""
    role: UserRole = UserRole.NONE
    currentProjectId: str = ""

    def isNull(self) -> bool:
        return self.uuid == UUID("{00000000-0000-0000-0000-000000000000}")

class Project(BaseModel):
    brief: Optional[str] = "Default"
    description: Optional[str] = ""
    id: Optional[int] = 0
    projects_defaults: Optional[int] = 0
    title: Optional[str] = ""
    uuid: Optional[UUID] = UUID("{00000000-0000-0000-0000-000000000000}")


class UserAuth(BaseModel):
    checkAuth: bool = False
    role: UserRole = UserRole.NONE



import pytest
from pycivil.EXAUtils.EXAExceptions import EXAExceptions as EXAExcept
from pycivil.EXAStructural.cheatsheets.codeNTC2018Rules import (
    BeamMinimumArea,
    BeamMinimumAreaInput,
    ConcreteMaterialInput,
    ConcreteMaterialOutput,
    RebarMaterialInput,
    RebarMaterialOutput,
    SolverBeamMinRebar,
    SolverConcrete,
    SolverSteelRebar,
)
from pycivil.EXAStructural.materials import Concrete, ConcreteSteel
from pydantic import ValidationError
from rich import print

VALID_CONCRETE_CODES = [
    (code, mat)
    for code, concretes in Concrete.tab_fck.items()
    for mat in concretes.keys()
]

INVALID_CONCRETE_CODES_MAT = [
    (code, mat + "_")
    for code, concretes in Concrete.tab_fck.items()
    for mat in concretes.keys()
]

INVALID_CONCRETE_CODES_CODE = [
    (code + "_", mat)
    for code, concretes in Concrete.tab_fck.items()
    for mat in concretes.keys()
]


def test_solver_concrete_raise() -> None:
    with pytest.raises(ValueError):
        SolverConcrete(ConcreteMaterialOutput())


@pytest.mark.parametrize(("code_key", "mat_key"), INVALID_CONCRETE_CODES_CODE)
def test_solver_concrete_code_invalid(code_key: str, mat_key: str) -> None:
    matIn = ConcreteMaterialInput()
    matIn.keyCode = code_key
    matIn.concreteClass = mat_key
    solver = SolverConcrete(matIn)
    with pytest.raises(ValueError):
        solver.run()


@pytest.mark.parametrize(("code_key", "mat_key"), INVALID_CONCRETE_CODES_MAT)
def test_solver_concrete_mat_invalid(code_key: str, mat_key: str) -> None:
    matIn = ConcreteMaterialInput()
    matIn.keyCode = code_key
    matIn.concreteClass = mat_key
    solver = SolverConcrete(matIn)
    with pytest.raises(EXAExcept):
        solver.run()


@pytest.mark.parametrize(("code_key", "mat_key"), VALID_CONCRETE_CODES)
def test_solver_concrete(code_key: str, mat_key: str) -> None:
    matIn = ConcreteMaterialInput()
    matIn.keyCode = code_key
    matIn.concreteClass = mat_key
    solver = SolverConcrete(matIn)
    assert solver.run()
    assert not solver.run(3)

    out = solver.getModelOutput()
    assert isinstance(out, ConcreteMaterialOutput)

    assert out.value_fck == float(mat_key[1 : mat_key.find("/")])
    assert out.value_Rck == float(mat_key[mat_key.find("/") + 1 :])


VALID_STEEL_CODES = [
    (code, mat)
    for code, steels in ConcreteSteel.tab_steel.items()
    for mat in steels.keys()
]

INVALID_STEEL_CODES_MAT = [
    (code, mat + "_")
    for code, steels in ConcreteSteel.tab_steel.items()
    for mat in steels.keys()
]

INVALID_STEEL_CODES_CODE = [
    (code + "_", mat)
    for code, steels in ConcreteSteel.tab_steel.items()
    for mat in steels.keys()
]


def test_solver_steel_raise() -> None:
    with pytest.raises(ValueError):
        SolverSteelRebar(ConcreteMaterialOutput())


@pytest.mark.parametrize(("code_key", "mat_key"), INVALID_STEEL_CODES_CODE)
def test_solver_steel_code_invalid(code_key: str, mat_key: str) -> None:
    matIn = RebarMaterialInput()
    matIn.keyCode = code_key
    matIn.steelClass = mat_key
    solver = SolverSteelRebar(matIn)
    with pytest.raises(ValueError):
        solver.run()


@pytest.mark.parametrize(("code_key", "mat_key"), INVALID_STEEL_CODES_MAT)
def test_solver_steel_mat_invalid(code_key: str, mat_key: str) -> None:
    matIn = RebarMaterialInput()
    matIn.keyCode = code_key
    matIn.steelClass = mat_key
    solver = SolverSteelRebar(matIn)
    with pytest.raises(EXAExcept):
        solver.run()


@pytest.mark.parametrize(("code_key", "mat_key"), VALID_STEEL_CODES)
def test_solver_steel(code_key: str, mat_key: str) -> None:
    matIn = RebarMaterialInput()
    matIn.keyCode = code_key
    matIn.steelClass = mat_key
    solver = SolverSteelRebar(matIn)
    assert solver.run()

    out = solver.getModelOutput()
    assert isinstance(out, RebarMaterialOutput)

    assert out.value_fyk == float(mat_key[1:4])


def test_beam_minimum_area_input():
    with pytest.raises(ValidationError):
        BeamMinimumAreaInput(hEl=-1.0)

    with pytest.raises(ValidationError):
        BeamMinimumAreaInput(hEl="-1.0")

    with pytest.raises(ValidationError):
        BeamMinimumAreaInput(rebarDComp="-1.0")

    with pytest.raises(ValidationError):
        BeamMinimumAreaInput(elementDescr=2)

    with pytest.raises(ValidationError):
        BeamMinimumAreaInput(hEl=100.0, cover=80.0, rebarD=21)

    with pytest.raises(ValidationError):
        BeamMinimumAreaInput(rebarD=-12)


def test_solver_beamMinimumArea() -> None:
    dataInput = BeamMinimumAreaInput()
    dataInput.keyCode = "NTC2018"
    dataInput.concreteClass = "C25/30"
    dataInput.steelClass = "B450C"

    # Test fake input model
    with pytest.raises(ValueError):
        SolverBeamMinRebar(RebarMaterialInput)

    solver = SolverBeamMinRebar(dataInput)
    assert solver.run()
    results = BeamMinimumArea(
        inputData=dataInput,
        logsData=solver.getModelLogs(),
        outputData=solver.getModelOutput(),
    )
    print("\n")
    print(results)



import math
import unittest
from pathlib import Path

from pycivil.EXAStructural.modeler import Point2d, SectionModeler
from pycivil.EXAStructural.plot import SectionPlot
from pycivil.EXAStructural.plot import SectionPlotView_Enum as ViewEnum


class Test(unittest.TestCase):
    def test_105_modeler_garbage(self):
        md = SectionModeler()
        md.setLogLevel(1)

        md.printInfo("AFTER CREATION")
        md.addSection(1, True)
        md.printInfo("AFTER ADD SECTION")
        md.addNode(1, 0, 0)
        md.addNode(2, 300, 0)
        md.addNode(3, 300, 600)
        md.addNode(4, 0, 600)
        md.getNodes()
        md.addTriangle(1, 1, 2, 3)
        md.addTriangle(2, 3, 4, 1)

        md.addNode(20, 60, 40)
        md.addNode(21, 120, 40)
        md.addNode(22, 180, 40)
        md.addNode(23, 240, 40)

        md.addCircle(20, 20, 28 / 2)
        md.addCircle(21, 21, 28 / 2)
        md.addCircle(22, 22, 28 / 2)
        md.addCircle(23, 23, 28 / 2)

        self.assertEqual(md.getCircles()[20].center.id, 20)

    def test_101_modeler(self):
        md = SectionModeler()
        md.printInfo("AFTER CREATION")

        md.setLogLevel(1)
        # test of addSection
        self.assertTrue(md.addSection(1, True))
        self.assertTrue(md.addSection(2, True))
        self.assertFalse(md.addSection(1, True))

        # test of setCurrent
        self.assertTrue(md.setCurrent(1))
        self.assertFalse(md.setCurrent(3))

        # test of getCurrent
        self.assertTrue(md.getCurrent())
        self.assertFalse(md.getCurrent() == 2)

        # test of getCurrentType
        self.assertTrue(md.getCurrentType() == "SectionModel")

        # test of getModelIndexes
        self.assertEqual(md.getModelIndices(), [1, 2])

        # test of sizeOfSectionModels
        self.assertEqual(md.sizeOfSectionModels(), 2)

        # test of addNode
        self.assertTrue(md.addNode(1, 0, 0))
        self.assertTrue(md.addNode(2, 300, 0))
        self.assertTrue(md.addNode(3, 299, 599))
        self.assertTrue(md.addNode(4, 0, 600))

        # test of nodesSize
        self.assertEqual(md.nodesSize(), 4)

        # test of getNodeX
        self.assertEqual(md.getNodeX(3), 299)
        self.assertRaises(RuntimeError, md.getNodeX, 11)

        # test of getNodeY
        self.assertEqual(md.getNodeY(3), 599)

        # test of setNodeX
        self.assertTrue(md.setNodeX(3, 300))
        self.assertTrue(md.setNodeY(3, 600))
        self.assertEqual(md.getNodeX(3), 300)
        self.assertEqual(md.getNodeY(3), 600)
        # test of getNodes
        nodes = md.getNodes()
        coords = ((0, 0), (300, 0), (300, 600), (0, 600))
        ids = (1, 2, 3, 4)
        for idn in nodes:
            idx = ids.index(idn)
            self.assertEqual(nodes[idn].xn, coords[idx][0])
            self.assertEqual(nodes[idn].yn, coords[idx][1])

        # test of addTriangle
        self.assertTrue(md.addTriangle(1, 1, 2, 3))
        self.assertTrue(md.addTriangle(2, 3, 4, 1))
        self.assertFalse(md.addTriangle(1, 1, 2, 3))
        self.assertFalse(md.addTriangle(3, 1, 2, 5))

        # test of getTriangles
        trianglesId = md.getTrianglesIds()
        trianglesCheck = ((1, 1, 2, 3), (2, 3, 4, 1))
        idx = 0
        for k in trianglesId:
            self.assertEqual(trianglesId[k][0], trianglesCheck[idx][1])
            self.assertEqual(trianglesId[k][1], trianglesCheck[idx][2])
            self.assertEqual(trianglesId[k][2], trianglesCheck[idx][3])
            idx += 1

        # test of trianglesSize
        self.assertEqual(md.trianglesSize(), 2)

        # test of addCircles
        self.assertTrue(md.addNode(20, 60, 40))
        self.assertTrue(md.addNode(21, 120, 40))
        self.assertTrue(md.addNode(22, 180, 40))
        self.assertTrue(md.addNode(23, 240, 40))
        self.assertTrue(md.addNode(10, 60, 560))
        self.assertTrue(md.addNode(11, 120, 560))
        self.assertTrue(md.addNode(12, 180, 560))
        self.assertTrue(md.addNode(13, 240, 560))

        self.assertTrue(md.addCircle(20, 20, 28 / 2))
        self.assertTrue(md.addCircle(21, 21, 28 / 2))
        self.assertTrue(md.addCircle(22, 22, 28 / 2))
        self.assertTrue(md.addCircle(23, 23, 28 / 2))
        self.assertTrue(md.addCircle(10, 10, 16 / 2))
        self.assertTrue(md.addCircle(11, 11, 16 / 2))
        self.assertTrue(md.addCircle(12, 12, 16 / 2))
        self.assertTrue(md.addCircle(13, 13, 16 / 2))

        self.assertFalse(md.addCircle(13, 13, 16 / 2))
        self.assertFalse(md.addCircle(13, -1, 16 / 2))

        # test of circlesSize
        self.assertEqual(md.circlesSize(), 8)

        # test of setCircleRadius
        oldRadius = md.getCircles()[20].radius
        self.assertTrue(md.setCircleRadius(20, 12 / 2))
        self.assertEqual(md.getCircles()[20].radius, 12 / 2)
        self.assertTrue(md.setCircleRadius(20, oldRadius))
        self.assertEqual(md.getCircles()[20].radius, oldRadius)

        # test of getTriangles
        circles = md.getCircles()
        circlesCheck = (
            (10, 10, 8),
            (11, 11, 8),
            (12, 12, 8),
            (13, 13, 8),
            (20, 20, 14),
            (21, 21, 14),
            (22, 22, 14),
            (23, 23, 14),
        )
        for idc in circlesCheck:
            self.assertEqual(circles[idc[0]].id, idc[0])
            self.assertEqual(circles[idc[0]].center.id, idc[1])
            self.assertEqual(circles[idc[0]].radius, idc[2])

        # test of addLawParaboleRectangle and addLawBilinear
        self.assertTrue(
            md.addLawParaboleRectangle(100, "C20/25", 20 / 1.5, 0.002, 0.0035)
        )
        self.assertTrue(md.addLawBilinear(200, "B450C", 450 / 1.15, 210000, 0.010))
        self.assertFalse(
            md.addLawParaboleRectangle(200, "C20/25", 20 / 1.5, 0.002, 0.0035)
        )

        # test of setTrianglesLaw setCirclesLaw
        self.assertTrue(md.setTrianglesLawInAllModels(100))
        self.assertTrue(md.setCirclesLawInAllModels(200))
        self.assertFalse(md.setTrianglesLawInAllModels(101))
        self.assertFalse(md.setCirclesLawInAllModels(101))

        # test of meshMake
        self.assertTrue(md.meshMake())

        # test of meshSize
        self.assertTrue(md.meshSize(), 2)

        # test of nodesSizeAtMesh, getNodesAtMesh,
        #         trianglesSizeAtMesh, getTrianglesAtMesh
        #         circlesSizeAtMesh
        nodesCheck = [
            [(0.0, 0.0), (300.0, 0.0), (300.0, 600.0)],
            [(300.0, 600.0), (0.0, 600.0), (0.0, 0.0)],
        ]
        trianglesCheck = [(1, 2, 3), (3, 4, 1)]
        for i in range(md.meshSize()):
            nbNodes = md.nodesSizeAtMesh(i)
            self.assertEqual(nbNodes, 3)

            nodes = md.getNodesAtMesh(i)
            for ii in range(nbNodes):
                node = nodes[ii]
                self.assertEqual(node.xn, nodesCheck[i][ii][0])
                self.assertEqual(node.yn, nodesCheck[i][ii][1])
                print(f"(id: *{node.id}* {node.xn},{node.yn})")
                del node

            nbTriangles = md.trianglesSizeAtMesh(i)
            self.assertEqual(nbTriangles, 1)

            trianglesIds = md.getTrianglesIdsAtMesh(i)
            for k in trianglesIds:
                triangle = trianglesIds[k]
                self.assertEqual(triangle[0], trianglesCheck[i][0])
                self.assertEqual(triangle[1], trianglesCheck[i][1])
                self.assertEqual(triangle[2], trianglesCheck[i][2])
                del triangle

            nbCircles = md.circlesSizeAtMesh(i)
            self.assertEqual(nbCircles, 0)

        self.assertEqual(md.calcSolidBarycenter(), Point2d(150, 300))

        # test of printInfo
        md.printInfo()

        # Need to do this because in getNodes(), getTriangles(), getCircles()
        # we make a copy of instances that doesn't destroy with garbage collector
        # ad ex. above using del
        #                  --------------------------
        #                        before garbage
        #                  --------------------------
        #                  Section models     ==> 2
        #                  Nodes              ==> 12
        #                  Triangles          ==> 2
        #                  Circles            ==> 8
        #                  Edges              ==> 0
        #                  Laws bilinear      ==> 1
        #                  Laws parabola rec. ==> 1
        #                  Deleting XStruModeler ...
        #                  --------------------------
        #                        after deleting
        #                  --------------------------
        #                  Section models     ==> 0
        #                  Nodes              ==> 0
        #                  Triangles          ==> 0
        #                  Circles            ==> 0
        #                  Edges              ==> 0
        #                  Laws bilinear      ==> 0
        #                  Laws parabola rec. ==> 0
        # del nodes
        # del circles
        # del triangles

    def test_102_modeler_calc(self):
        # Build modeler
        md = SectionModeler()

        # Add section and make it current
        md.addSection(1, True)

        # Add node for section
        md.addNode(1, 0, 0)
        md.addNode(2, 300, 0)
        md.addNode(3, 300, 600)
        md.addNode(4, 0, 600)

        md.addTriangle(1, 1, 2, 3)
        md.addTriangle(2, 3, 4, 1)

        # Add node and rebar on bottom
        md.addNode(20, 60, 40)
        md.addNode(21, 120, 40)
        md.addNode(22, 180, 40)
        md.addNode(23, 240, 40)

        diameterBot = 28
        md.addCircle(20, 20, diameterBot / 2)
        md.addCircle(21, 21, diameterBot / 2)
        md.addCircle(22, 22, diameterBot / 2)
        md.addCircle(23, 23, diameterBot / 2)

        # Add node and rebar on top
        md.addNode(10, 60, 560)
        md.addNode(11, 120, 560)
        md.addNode(12, 180, 560)
        md.addNode(13, 240, 560)

        diameterTop = 16
        md.addCircle(10, 10, diameterTop / 2)
        md.addCircle(11, 11, diameterTop / 2)
        md.addCircle(12, 12, diameterTop / 2)
        md.addCircle(13, 13, diameterTop / 2)

        solidBary = md.calcSolidBarycenter()
        print(f"solidBary = {solidBary}")
        self.assertEqual(solidBary.x, 150.0)
        self.assertEqual(solidBary.y, 300.0)

        pointBary = md.calcPointBarycenter(area=False)
        print(f"pointBary unary = {pointBary}")
        self.assertAlmostEqual(pointBary.x, 150.0)
        self.assertAlmostEqual(pointBary.y, 300.0)

        pointBary = md.calcPointBarycenter(area=True)
        print(f"pointBary with area = {pointBary}")

        areaTop = 4 * math.pow(diameterTop, 2) * math.pi / 4
        yTop = 600 - 40
        areaBot = 4 * math.pow(diameterBot, 2) * math.pi / 4
        yBot = 40
        yg = (areaTop * yTop + areaBot * yBot) / (areaTop + areaBot)
        self.assertAlmostEqual(pointBary.y, yg)

        # TODO: Need to be fixed calc in exagone
        md.calcSolidAreaProperties()

        self.assertTrue(areaTop + areaBot, md.calcPointArea())
        self.assertTrue(300 * 600, md.calcSolidArea())

        md.printInfo()

    def test_103_modeler_plot(self):
        # Build modeler
        md = SectionModeler()

        # Add section and make it current
        md.addSection(1, True)

        # Add node for section
        md.addNode(1, 0, 0)
        md.addNode(2, 300, 0)
        md.addNode(3, 300, 600)
        md.addNode(4, 0, 600)

        md.addTriangle(1, 1, 2, 3)
        md.addTriangle(2, 3, 4, 1)

        # Add node and rebar on bottom
        md.addNode(20, 60, 40)
        md.addNode(21, 120, 40)
        md.addNode(22, 180, 40)
        md.addNode(23, 240, 40)

        diameterBot = 28
        md.addCircle(20, 20, diameterBot / 2)
        md.addCircle(21, 21, diameterBot / 2)
        md.addCircle(22, 22, diameterBot / 2)
        md.addCircle(23, 23, diameterBot / 2)

        # Add node and rebar on top
        md.addNode(10, 60, 560)
        md.addNode(11, 120, 560)
        md.addNode(12, 180, 560)
        md.addNode(13, 240, 560)

        diameterTop = 16
        md.addCircle(10, 10, diameterTop / 2)
        md.addCircle(11, 11, diameterTop / 2)
        md.addCircle(12, 12, diameterTop / 2)
        md.addCircle(13, 13, diameterTop / 2)

        sp = SectionPlot()
        sp.modeler = md
        sp.setView(ViewEnum.SP_VIEW_SECTION)
        sp.plot()
        sp.save(Path(__file__).parent / "test_103_modeler_plot_1.png")
        sp.setView(ViewEnum.SP_VIEW_TRIANGLES)
        sp.plot()
        sp.save(Path(__file__).parent / "test_103_modeler_plot_2.png")
        sp.setView(ViewEnum.SP_VIEW_SECTION_NOID)
        sp.plot()
        sp.save(Path(__file__).parent / "test_103_modeler_plot_3.png")
        sp.setView(ViewEnum.SP_VIEW_TRIANGLE_NOID)
        sp.plot()
        sp.save(Path(__file__).parent / "test_103_modeler_plot_4.png")
        sp.setView(ViewEnum.SP_VIEW_SECTION)
        sp.setView(ViewEnum.SP_VIEW_GRID)
        sp.plot()
        sp.save(Path(__file__).parent / "test_103_modeler_plot_5.png")
        sp.setView(ViewEnum.SP_VIEW_TRIANGLES)
        sp.setView(ViewEnum.SP_HIDE_NODES)
        sp.plot()
        sp.save(Path(__file__).parent / "test_103_modeler_plot_6.png")
        # sp.show()
        md.setLogLevel(1)
        del md

    # --------------------
    # BY-HAND TEST CASE #1
    # --------------------
    def test_104_modeler_calc(self):
        # Build modeler
        md = SectionModeler()

        # Add section and make it current
        md.addSection(1, True)

        # Add node for section
        md.addNode(1, 0, 0)
        md.addNode(2, 300, 0)
        md.addNode(3, 300, 600)
        md.addNode(4, 0, 600)

        md.addTriangle(1, 1, 2, 3)
        md.addTriangle(2, 3, 4, 1)

        # Add node and rebar on bottom
        md.addNode(20, 60, 54)
        md.addNode(21, 120, 54)
        md.addNode(22, 180, 54)
        md.addNode(23, 240, 54)

        diameterBot = 28
        md.addCircle(20, 20, diameterBot / 2)
        md.addCircle(21, 21, diameterBot / 2)
        md.addCircle(22, 22, diameterBot / 2)
        md.addCircle(23, 23, diameterBot / 2)

        # Add node and rebar on top
        md.addNode(10, 60, 550)
        md.addNode(11, 120, 550)
        md.addNode(12, 180, 550)
        md.addNode(13, 240, 550)

        diameterTop = 20
        md.addCircle(10, 10, diameterTop / 2)
        md.addCircle(11, 11, diameterTop / 2)
        md.addCircle(12, 12, diameterTop / 2)
        md.addCircle(13, 13, diameterTop / 2)

        md.nCoeff = 15

        solidBary = md.calcSolidBarycenter()
        print("\n")
        print(f"solidBary = ({solidBary.x:.3f}, {solidBary.y:.3f})")
        self.assertEqual(solidBary.x, 150.0)
        self.assertEqual(solidBary.y, 300.0)

        pointBary = md.calcPointBarycenter()
        print(f"pointBary = ({pointBary.x:.3f}, {pointBary.y:.3f})")
        self.assertAlmostEqual(pointBary.x, 150.0)
        self.assertAlmostEqual(pointBary.y, 221.6, delta=0.1)

        idealBary = md.calcIdealBarycenter()
        print(f"idealBary = ({idealBary.x:.3f}, {idealBary.y:.3f})")
        self.assertAlmostEqual(idealBary.x, 150.0)
        self.assertAlmostEqual(idealBary.y, 281.38, delta=0.1)

        solidArea = md.calcSolidArea()
        print(f"solidArea = {solidArea:.3f}")
        self.assertAlmostEqual(solidArea, 180.0e3, delta=0.1)

        pointArea = md.calcPointArea()
        print(f"pointArea = {pointArea:.3f}")
        self.assertAlmostEqual(pointArea, 3.72e3, delta=1.0)

        idealArea = md.calcIdealArea()
        print(f"idealArea = {idealArea:.3f}")
        self.assertAlmostEqual(idealArea, 235.8e3, delta=6.0)

        md.setLogLevel(1)

    def test_106_modeler_plot_mesh(self):
        # Build modeler
        md = SectionModeler()

        # Add section and make it current
        md.addSection(1, True)

        # Add node for section
        md.addNode(1, 0, 0)
        md.addNode(2, 300, 0)
        md.addNode(3, 300, 600)
        md.addNode(4, 0, 600)

        md.addTriangle(1, 1, 2, 3)
        md.addTriangle(2, 3, 4, 1)

        # Add node and rebar on bottom
        md.addNode(20, 60, 40)
        md.addNode(21, 120, 40)
        md.addNode(22, 180, 40)
        md.addNode(23, 240, 40)

        diameterBot = 28
        md.addCircle(20, 20, diameterBot / 2)
        md.addCircle(21, 21, diameterBot / 2)
        md.addCircle(22, 22, diameterBot / 2)
        md.addCircle(23, 23, diameterBot / 2)

        # Add node and rebar on top
        md.addNode(10, 60, 560)
        md.addNode(11, 120, 560)
        md.addNode(12, 180, 560)
        md.addNode(13, 240, 560)

        diameterTop = 16
        md.addCircle(10, 10, diameterTop / 2)
        md.addCircle(11, 11, diameterTop / 2)
        md.addCircle(12, 12, diameterTop / 2)
        md.addCircle(13, 13, diameterTop / 2)

        md.meshMake()

        md.meshesSliceAtYrays([100, 200, 300, 400, 500])

        sp = SectionPlot()
        sp.modeler = md
        sp.setView(ViewEnum.SP_VIEW_MESH)
        sp.plot()
        sp.save(Path(__file__).parent / "test_106_modeler_plot_mesh_slice_yray.png")

        md.meshReset()
        sp.plot()
        sp.save(Path(__file__).parent / "test_106_modeler_plot_mesh_resetted.png")

        md.meshesSliceAtRays(
            [
                [0, 1, -100],
                [0, 1, -200],
                [0, 1, -300],
                [0, 1, -400],
                [0, 1, -500],
            ]
        )

        sp.plot()
        sp.save(Path(__file__).parent / "test_106_modeler_plot_mesh_slice_ray_1.png")

        md.meshReset()
        md.meshesSliceAtRays(
            [
                [-1, 1, -2000],
                [-1, 1, -1500],
                [-1, 1, -1000],
            ]
        )

        sp.plot()
        sp.save(Path(__file__).parent / "test_106_modeler_plot_mesh_slice_ray_2.png")

        md.setLogLevel(1)
        del md

    def test_107_modeler_geometry(self):
        # Build modeler
        md = SectionModeler()

        # Add section and make it current
        md.addSection(1, True)

        # Add node for section
        md.addNode(1, 0, 0)
        md.addNode(2, 300, 0)
        md.addNode(3, 300, 600)
        md.addNode(4, 0, 600)

        md.addTriangle(1, 1, 2, 3)
        md.addTriangle(2, 3, 4, 1)

        # Add node and rebar on bottom
        md.addNode(20, 60, 40)
        md.addNode(21, 120, 40)
        md.addNode(22, 180, 40)
        md.addNode(23, 240, 40)

        diameterBot = 28
        md.addCircle(20, 20, diameterBot / 2)
        md.addCircle(21, 21, diameterBot / 2)
        md.addCircle(22, 22, diameterBot / 2)
        md.addCircle(23, 23, diameterBot / 2)

        # Add node and rebar on top
        md.addNode(10, 60, 560)
        md.addNode(11, 120, 560)
        md.addNode(12, 180, 560)
        md.addNode(13, 240, 560)

        diameterTop = 16
        md.addCircle(10, 10, diameterTop / 2)
        md.addCircle(11, 11, diameterTop / 2)
        md.addCircle(12, 12, diameterTop / 2)
        md.addCircle(13, 13, diameterTop / 2)

        solidBary = md.calcSolidBarycenter()
        print(f"solidBary = {solidBary}")
        self.assertEqual(solidBary.x, 150.0)
        self.assertEqual(solidBary.y, 300.0)

        md.saveGeometry()
        md.moveToSolidBarycenter()
        solidMoved = md.calcSolidBarycenter()
        print(f"solidMoved = {solidMoved}")
        self.assertEqual(solidMoved.x, 0.0)
        self.assertEqual(solidMoved.y, 0.0)

        md.restoreGeometry()
        solidBary = md.calcSolidBarycenter()
        print(f"solidBary restored = {solidBary}")
        self.assertEqual(solidBary.x, 150.0)
        self.assertEqual(solidBary.y, 300.0)

        del md


if __name__ == "__main__":
    unittest.main()



from pycivil import EXAGeometry as eg
from pycivil.EXAStructural import codes, materials, sections
from pycivil.EXAStructural import plot as ep

# import numpy as np
# from matplotlib.path import Path
# from matplotlib.patches import PathPatch
# import matplotlib.pyplot as plt
# from matplotlib.ticker import FormatStrFormatter

# ******************* CONCRETE SECTION first method

# Setting working code
code_EC2 = codes.Code("EC2")
print(code_EC2)

# Setting concrete material
cls_material = materials.Concrete("EC2_C20/25")
cls_material.setByCode(code_EC2, "C20/25")
print(cls_material)

# Setting concrete shape
rect_shape = eg.ShapeRect(300, 600)
print(rect_shape)

rectangularSection = sections.StructSectionItem(rect_shape, cls_material)
print(rectangularSection)

# Setting steel material
steel_material = materials.ConcreteSteel("EC2_450C")
steel_material.setByCode(code_EC2, "B450C")
print(steel_material)

# Retriving special points
MB = rectangularSection.getShape().getShapePoint("MB")
MT = rectangularSection.getShape().getShapePoint("MT")

# Setting steel shape
area_shape_1 = eg.ShapeArea(2461.76)
area_shape_1.setOrigin(MB + eg.Point2d(0, 50))
steel_1 = sections.StructSectionItem(area_shape_1, steel_material)
print(steel_1)

area_shape_2 = eg.ShapeArea(803.84)
area_shape_2.setOrigin(MT + eg.Point2d(0, -50))
steel_2 = sections.StructSectionItem(area_shape_2, steel_material)

# TODO: __str__ for ConcreteSection
myfirstsection = sections.ConcreteSection(1, "300x500 EC2")
myfirstsection.setStructConcrItem(rectangularSection)
myfirstsection.setStructSteelItems([steel_1, steel_2])

print(myfirstsection)

NxMz, Fields, NxMzBoundingRect = myfirstsection.build2dInteractionCompleteDomain(
    negative_compression=True
)

strDimension = "section 300x600 - Ai=2461.76 As=803.84"
strMaterial = "concrete: B450C steel: C20/25"

tensionPoints = [[-300 * 1e3, +300 * 1e6], [-400 * 1e3, +400 * 1e6]]

ep.interactionDomainBasePlot2d(
    NxMz,
    Fields,
    xLabel="Nx [KN]",
    yLabel="Mz [KN*m]",
    titleAddStr=strDimension + "\n" + strMaterial,
    tensionPoints=tensionPoints,
)



# Drawing interation domain and evidenziate intersection on plot diagram

from pycivil.EXAStructural import templateRCRect as est

# Setting new instance of section with
# id = 1 and name = " testing intersection"
section = est.RCTemplRectEC2(1, "First Section testing intersection")

section.setDimH(500.0)
section.setDimW(300.0)

# 'MB' means medium bottom area
# 'MT' means medium top area
section.addSteelArea("MB", 40.0, 1000.0)
section.addSteelArea("MT", 40.0, 0000.0)

section.setMaterials("C32/40", "B450C", 0.6 * 32, 0.8 * 450, 15.0)
# Adding Tension Point

section.getMaterialConcr().set_alphacc(1.00)

print(section)

# pt = section.addTensionPoint2d(N = 5.000E+05, M = 2.103E+08)
# p0 = section.addTensionPoint2d(N = 000.0*1E+3, M = 000.0*1E+6)

# center = p0

# Building a set points of interaction diagramm
# pointsCloud, bounding = section.interactionDomainBuild2d(nbPoints = 50, bounding = True, SLS = False)
pointsCloud, bounding = section.interactionDomainBuild2d(
    nbPoints=50, bounding=True, SLS=True
)

section.addSteelArea("MB", 40.0, 2000.0)
section.addSteelArea("MT", 40.0, 0000.0)

pointsCloud, bounding = section.interactionDomainBuild2d(
    nbPoints=50, bounding=True, SLS=True
)

section.addSteelArea("MB", 40.0, 4000.0)
section.addSteelArea("MT", 40.0, 0000.0)

pointsCloud, bounding = section.interactionDomainBuild2d(
    nbPoints=50, bounding=True, SLS=True
)

section.addSteelArea("MB", 40.0, 6000.0)
section.addSteelArea("MT", 40.0, 0000.0)

pointsCloud, bounding = section.interactionDomainBuild2d(
    nbPoints=50, bounding=True, SLS=True
)

brox = bounding[1] - bounding[0]
broy = bounding[3] - bounding[2]

# contained, pintersect, intfactor = pointsCloud.contains(pt.x,pt.y, rayFromCenter = True, center = p0, ro = [brox,broy], positiveFactor = True)

linesOfMinimalPoints = []

"""
if len(pintersect)==2:
    # section.addTensionPoint2d(pintersect[0])
    # section.addTensionPoint2d(pintersect[1])
    linesOfMinimalPoints.append([pt,pintersect[0]])
    linesOfMinimalPoints.append([pt,pintersect[1]])
    print("intfactor = %1.6f"%intfactor)
    print("Punti di intersezione ....................")
    print(pintersect[0])
    print(pintersect[1])
"""
# section.addTensionPoint2d(center)

section.interactionDomainPlot2d()

# print(section.getInteractionDomain()[0])

# section.interactionDomainPlot2d(lines = linesOfMinimalPoints)
"""
print("Contained = %1.0f"%contained)
print("Punto tensione ....................")
print(pt)
print("Centro ....................")
print(center)
"""



"""A object-oriented library for Structural Engineers.

It can be used every day and extended step by step by engineer.
Typical use is for production, development and testing.

For production, you should use latest version of package writing code
with a nootebook like Jupyter or Marimo.
```
from pycivil.<package-name>.<module-name> import <class-name> as <alias>
```
Typical usage example:
```
from pycivil.EXAStructural.loads import ForcesOnSection as Forces
```

Packages list:

  1. EXAGeometry
  2. EXAGeotechnical
  3. EXAMicroServices
  4. EXAParametric
  5. EXAStructural
  6. EXAStructuralCheckable
  7. EXAStructuralModel
  8. EXAUtils
"""


from pathlib import Path
import pytest

from pycivil.EXAStructural.cheatsheets.codeNTC2018Rules import (
    BeamMinimumArea,
    BeamMinimumAreaInput,
    BeamMinimumAreaOutput,
    BeamMinimumAreaLogs,
    ConcreteMaterial,
    ConcreteMaterialInput,
    ConcreteMaterialOutput,
    RebarMaterial,
    RebarMaterialInput,
    RebarMaterialOutput,
    SolverBeamMinRebar,
    SolverConcrete,
    SolverSteelRebar,
)
from pycivil.EXAUtils import latexCheatSheets
from pycivil.EXAUtils.report import (
    Fragment,
    Reporter,
    ReportTemplateEnum,
    getTemplatesPath,
)

@pytest.mark.needLatex
def test_concrete_by_law(tmp_path: Path):
    matInput = ConcreteMaterialInput()
    solver = SolverConcrete(inputModel=matInput)

    assert solver.run()

    outputData = solver.getModelOutput()
    assert isinstance(outputData, ConcreteMaterialOutput)
    material = ConcreteMaterial(inputData=matInput, outputData=outputData)

    latexTemplatePath = getTemplatesPath()

    print(f"Latex Templates path for test is {latexTemplatePath}")
    fragBuilder = latexCheatSheets.ConcreteMaterialCS(
        latexTemplatePath=latexTemplatePath, data=material
    )

    frag_title = Fragment()
    frag_title.add(line=r"\section{Calcestruzzo adottato per l'elemento}")
    frag = fragBuilder.buildFragment()

    reporter = Reporter(latexTemplatePath)
    reporter.linkFragments(
        template=ReportTemplateEnum.TEX_ENG_CAL, fragments=[frag_title, frag]
    )
    reporter.compileDocument(path=str(tmp_path), fileName="TEX_ENG_CAL")
    print(f"Temporary path for test is {tmp_path}")
    assert tmp_path.joinpath("TEX_ENG_CAL.pdf").exists()
    assert tmp_path.joinpath("TEX_ENG_CAL.tex").exists()

    reporter.linkFragments(
        template=ReportTemplateEnum.TEX_KOMA, fragments=[frag_title, frag]
    )
    reporter.compileDocument(path=str(tmp_path), fileName="TEX_KOMA")
    print(f"Temporary path for test is {tmp_path}")
    assert tmp_path.joinpath("TEX_KOMA.pdf").exists()
    assert tmp_path.joinpath("TEX_KOMA.tex").exists()

    reporter.linkFragments(
        template=ReportTemplateEnum.TEX_MAIN,
        main_file_name="sheet-MatConcrete-NTC2018-ita-main.tex",
        fragments=[frag],
    )
    reporter.compileDocument(path=str(tmp_path), fileName="TEX_MAIN")
    print(f"Temporary path for test is {tmp_path}")
    assert tmp_path.joinpath("TEX_MAIN.pdf").exists()
    assert tmp_path.joinpath("TEX_MAIN.tex").exists()

@pytest.mark.needLatex
def test_steelRebar_by_law(tmp_path: Path):
    matInput = RebarMaterialInput()
    solver = SolverSteelRebar(inputModel=matInput)

    assert solver.run()
    outputData = solver.getModelOutput()
    assert isinstance(outputData, RebarMaterialOutput)
    material = RebarMaterial(inputData=matInput, outputData=outputData)

    latexTemplatePath = getTemplatesPath()
    print(f"Latex Templates path for test is {latexTemplatePath}")
    fragBuilder = latexCheatSheets.RebarMaterialCS(
        latexTemplatePath=latexTemplatePath, data=material
    )
    frag_title = Fragment()
    frag_title.add(line=r"\section{Acciaio adottato per l'elemento}")
    frag = fragBuilder.buildFragment()

    reporter = Reporter(latexTemplatePath)
    reporter.linkFragments(
        template=ReportTemplateEnum.TEX_ENG_CAL, fragments=[frag_title, frag]
    )
    reporter.compileDocument(path=str(tmp_path), fileName="TEX_ENG_CAL")
    print(f"Temporary path for test is {tmp_path}")
    assert tmp_path.joinpath("TEX_ENG_CAL.pdf").exists()
    assert tmp_path.joinpath("TEX_ENG_CAL.tex").exists()

    reporter.linkFragments(
        template=ReportTemplateEnum.TEX_KOMA, fragments=[frag_title, frag]
    )
    reporter.compileDocument(path=str(tmp_path), fileName="TEX_KOMA")
    print(f"Temporary path for test is {tmp_path}")
    assert tmp_path.joinpath("TEX_KOMA.pdf").exists()
    assert tmp_path.joinpath("TEX_KOMA.tex").exists()

    reporter.linkFragments(
        template=ReportTemplateEnum.TEX_MAIN,
        main_file_name="sheet-MatRebar-NTC2018-ita-main.tex",
        fragments=[frag],
    )
    reporter.compileDocument(path=str(tmp_path), fileName="TEX_MAIN")
    print(f"Temporary path for test is {tmp_path}")
    assert tmp_path.joinpath("TEX_MAIN.pdf").exists()
    assert tmp_path.joinpath("TEX_MAIN.tex").exists()

@pytest.mark.needLatex
def test_beam_minimum_area_default(tmp_path: Path):
    dataInput = BeamMinimumAreaInput()
    solver = SolverBeamMinRebar(inputModel=dataInput)

    assert solver.run()

    logsData = solver.getModelLogs()
    assert isinstance(logsData, BeamMinimumAreaLogs)

    outputData = solver.getModelOutput()
    assert isinstance(outputData, BeamMinimumAreaOutput)

    data = BeamMinimumArea(
        inputData=dataInput,
        logsData=logsData,
        outputData=outputData,
    )

    latexTemplatePath = getTemplatesPath()

    print(f"Latex Templates path for test is {latexTemplatePath}")
    fragBuilder = latexCheatSheets.BeamMinimumAreaCS(
        latexTemplatePath=latexTemplatePath, data=data
    )

    frag_title = Fragment()
    frag_title.add(line=r"\section{Area minima delle travi secondo NTC2018}")
    frag = fragBuilder.buildFragment()

    reporter = Reporter(latexTemplatePath)
    reporter.linkFragments(
        template=ReportTemplateEnum.TEX_ENG_CAL, fragments=[frag_title, frag]
    )
    reporter.compileDocument(path=str(tmp_path), fileName="TEX_ENG_CAL")
    print(f"Temporary path for test is {tmp_path}")
    assert tmp_path.joinpath("TEX_ENG_CAL.pdf").exists()
    assert tmp_path.joinpath("TEX_ENG_CAL.tex").exists()

    reporter.linkFragments(
        template=ReportTemplateEnum.TEX_KOMA, fragments=[frag_title, frag]
    )
    reporter.compileDocument(path=str(tmp_path), fileName="TEX_KOMA")
    print(f"Temporary path for test is {tmp_path}")
    assert tmp_path.joinpath("TEX_KOMA.pdf").exists()
    assert tmp_path.joinpath("TEX_KOMA.tex").exists()
    reporter.linkFragments(
        template=ReportTemplateEnum.TEX_MAIN,
        main_file_name="sheet-BeamMinimumArea-NTC2018-ita-main.tex",
        fragments=[frag],
    )
    reporter.compileDocument(path=str(tmp_path), fileName="TEX_MAIN")
    print(f"Temporary path for test is {tmp_path}")
    assert tmp_path.joinpath("TEX_MAIN.pdf").exists()
    assert tmp_path.joinpath("TEX_MAIN.tex").exists()



import copy
import pickle
from pathlib import Path
from typing import Any, List, Optional, Tuple, Union
from warnings import warn

from pydantic import BaseModel

from pycivil.EXAGeometry.geometry import Point2d, Point2dMdl, Seg2d, Vector2d


class PointCloud2dMdl(BaseModel):
    points: Optional[List[Point2dMdl]] = None


class PointCloud2d:
    def __init__(self, *args):

        if len(args) == 0:
            self.__points: List[Point2d] = []
            self.__metaData: List[List[Any]] = []
            self.__signs: List[float] = []

        elif len(args) == 1 and isinstance(args[0], list):
            if len(args[0]) == 0:
                self.__points: List[Point2d] = []
                self.__metaData: List[List[Any]] = []
                self.__signs: List[float] = []
            else:
                self.__points: List[Point2d] = [Point2d()] * len(args[0])
                self.__signs: List[float] = [0.0] * (len(args[0]) - 2)
                if isinstance(args[0][0], list):
                    self.__metaData: List[List[Any]] = [[]] * len(args[0])
                    for idx in range(len(self.__points)):
                        metaLenght = len(args[0][idx])
                        assert metaLenght >= 2

                        assert isinstance(args[0][idx][0], (float, int))
                        assert isinstance(args[0][idx][1], (float, int))
                        self.__points[idx] = Point2d(args[0][idx][0], args[0][idx][1])

                        if metaLenght >= 2:
                            self.__metaData[idx] = args[0][idx][2 : (metaLenght + 1)]

                        if idx >= 2:
                            vecStart = Vector2d(
                                self.__points[idx - 2], self.__points[idx - 1]
                            )
                            vecEnd = Vector2d(
                                self.__points[idx - 1], self.__points[idx]
                            )
                            self.__signs[idx - 2] = vecStart.cross(vecEnd)

                elif isinstance(args[0][0], Point2d):
                    self.__metaData: List[List[Any]] = []
                    for idx in range(len(self.__points)):
                        assert isinstance(args[0][idx], Point2d)
                        self.__points[idx] = args[0][idx]
                        if idx >= 2:
                            vecStart = Vector2d(
                                self.__points[idx - 2], self.__points[idx - 1]
                            )
                            vecEnd = Vector2d(
                                self.__points[idx - 1], self.__points[idx]
                            )
                            self.__signs[idx - 2] = vecStart.cross(vecEnd)
                else:
                    raise ValueError("Only list or Point2d for first arg !!!")

        elif len(args) == 1 and isinstance(args[0], PointCloud2dMdl):
            self.__metaData: List[List[Any]] = []
            self.__signs: List[float] = []
            self.__points: List[Point2d] = []
            if args[0].points is not None:
                for p in args[0].points:
                    self.__points.append(Point2d(x=p.xy[0], y=p.xy[1]))
        else:
            raise ValueError("Only one arg list or PointCloud2dMdl !!!")

        self.__index = 0

    def __iter__(self):
        return self

    def __next__(self):
        if self.__index < len(self.__points):
            item = self.__points[self.__index]
            self.__index += 1
            return item
        else:
            raise StopIteration

    def __getitem__(self, key):
        return self.__points[key]

    def __str__(self):
        dispstr = self.__points.__str__()
        return dispstr

    def __len__(self):
        return len(self.__points)

    def isConvex(self) -> bool:
        if len(self.__signs) > 0:
            s = self.__signs[0] > 0
            for i in range(1, len(self.__signs)):
                if (self.__signs[i] > 0) != s:
                    return False
            return True
        print(
            "WARNING: Can't know about convexity because lenght is null !!!. Return False"
        )
        return False

    def __eq__(self, __value) -> bool:
        if len(__value) != len(self):
            return False
        for idx in range(len(__value)):
            if self.__points[idx] != __value[idx]:
                return False
        return True

    def getPoints(self):
        return self.__points

    def getMetaAt(self, idx: int) -> List[Any]:
        return self.__metaData[idx]

    def boundingBox(self) -> Tuple[float, float, float, float]:
        bb = [0.0, 0.0, 0.0, 0.0]
        for p in self.__points:
            if p.x < bb[0]:
                bb[0] = p.x
            if p.x > bb[1]:
                bb[1] = p.x
            if p.y < bb[2]:
                bb[2] = p.y
            if p.y > bb[3]:
                bb[3] = p.y

        return bb[0], bb[1], bb[2], bb[3]

    def contains(
        self,
        xp: float,
        yp: float,
        ro: Tuple[float, float],
        convex: bool = True,
        rayFromCenter=True,
        center: Union[Point2d, None] = None,
    ):

        if center is None:
            center = Point2d(0.0, 0.0)

        if not isinstance(xp, float) and not isinstance(yp, int):
            raise ValueError("Only float or int type for N!!!")

        if not isinstance(xp, float) and not isinstance(yp, int):
            raise ValueError("Only float or int type for M!!!")

        if not convex:
            raise ValueError("Only algorithm for convex shape !!!")

        if len(self.__points) < 3:
            raise ValueError("Containts at least 3 points !!!")

        p = Point2d(xp, yp)

        # ro useful for intersection point. Normally ro is estimed
        # with boudingbox calculated during forming interaction
        # domain.
        rox = ro[0]
        roy = ro[1]

        contained: bool = True
        pintersect: List[Point2d] = []
        pintersectFactor: float = 0.0
        pintersectFactors: List[float] = []
        pindex: List[int] = []  # Index that intersection found

        p1 = self.__points[0]
        p2 = self.__points[0 + 1]
        v2 = Vector2d(p2, p1)
        v1 = Vector2d(p1, p)

        minLenght = v1.norm_roxroy(rox, roy)
        if rayFromCenter:
            normro = False
            seg = Seg2d(p2, p1)
            ray = Seg2d(p, center)
            hasIntersection, pint = seg.intersectAndContaints(ray)
            if hasIntersection:
                pintersect.append(pint)
                pintersectFactors.append(ray.extensionFactor(pint))
                pindex.append(0)
        else:
            normro = True

        minPoint = p1
        cross = v1.cross(v2)
        check_contained = True
        for i, _val in enumerate(self.__points[0 : len(self.__points) - 2], 1):
            p1 = self.__points[i]
            p2 = self.__points[i + 1]
            v2 = Vector2d(p2, p1)
            v1 = Vector2d(p1, p)
            if normro:
                minLenght_i = v1.norm_roxroy(rox, roy)
                if minLenght_i < minLenght:
                    minLenght = minLenght_i
                    minPoint = p1
            elif rayFromCenter:
                # if (p2 != p1):
                seg = Seg2d(p2, p1)
                ray = Seg2d(p, center)
                hasIntersection, pint = seg.intersectAndContaints(ray)
                if hasIntersection:
                    found = False
                    for ii in pintersect:
                        # Relative precision for excluding nearest points for
                        # some case precision issue
                        # We can not use i==pint
                        same_point = ii.isEqualTo(pint, rox, roy, 1e-15)
                        if same_point:
                            found = True
                    if not found:
                        pintersect.append(pint)
                        pintersectFactors.append(ray.extensionFactor(pint))
                        pindex.append(i)
            else:
                minLenght_i = v1.norm()
                if minLenght_i < minLenght:
                    minLenght = minLenght_i
                    minPoint = p1

            cross_new = v1.cross(v2)
            if cross * cross_new < 0 and check_contained:
                contained = False
                check_contained = False
            cross = cross_new

        pindexn = -1
        d1 = 0
        d2 = 0
        d3 = 0

        if normro:
            warn("normro not tested !!!")
            return contained, minLenght, minPoint, None

        if len(pintersect) != 2:
            print(
                f"WARNING: intersection with point P=({xp}, {yp}) are {len(pintersect)} not 2 !!!"
            )
            print(f"         seg P1=({p1.x}, {p1.y})-P2=({p2.x}, {p2.y})")
            for p in pintersect:
                print(p)
            return contained, pintersect, pintersectFactor, [pindexn, d1, d2, d3]

        if pintersectFactors[0] < 0:
            pintersectFactor = pintersectFactors[1]
            pindexn = pindex[1]
            pleft = self.__points[pindex[1]]
            pright = self.__points[pindex[1] + 1]
            d1 = pintersect[1].distance(pleft)
            d2 = pintersect[1].distance(pright)
            d3 = pleft.distance(pright)
            ptemp = copy.deepcopy(pintersect)
            pintersect[0] = ptemp[1]
            pintersect[1] = ptemp[0]
        else:
            pintersectFactor = pintersectFactors[0]
            pindexn = pindex[0]
            pleft = self.__points[pindex[0]]
            pright = self.__points[pindex[0] + 1]
            d1 = pintersect[0].distance(pleft)
            d2 = pintersect[0].distance(pright)
            d3 = pleft.distance(pright)

        if pintersectFactor < 1.0:
            contained = False
        else:
            contained = True

        return contained, pintersect, pintersectFactor, [pindexn, d1, d2, d3]

    def save(self, file: Path):
        toSave = {"points": self.__points, "metaData": self.__metaData}
        with open(file, "bw") as f:
            pickle.dump(toSave, f)

    def open(self, file: Path):
        with open(file, "br") as f:
            toRead = pickle.load(f)
            self.__points = toRead["points"]
            self.__metaData = toRead["metaData"]

    def model(self) -> PointCloud2dMdl:
        points: List[Point2dMdl] = [
            Point2dMdl(x=i, y=i, null=True) for i in range(len(self.__points))
        ]
        for i, p in enumerate(points):
            p.xy = (self.__points[i].x, self.__points[i].y)
        return PointCloud2dMdl(points=points)



from pycivil.EXAUtils.ucasefe import RCRectCalculator

def test_rc_rect_calculator_001(tmp_path):
    calc = RCRectCalculator("test", "sezione 1")
    calc.setDescription("sezione 1")
    calc.setLogLevel(3)
    calc.setJobPath(str(tmp_path))

    KN = 1000
    KNm = 1000000

    calc.setDimensions(300, 600)

    calc.addForce(N= 150*KN, M= 150*KNm, T=150*KN, descr="force_1",
                  limit_state="serviceability", frequency="characteristic",
                  check_required=["SLE-NM"])
    calc.addForce(N= 100*KN, M= 145*KNm, T=120*KN, descr="force_2",
                  limit_state="serviceability", frequency="quasi-permanent",
                  check_required=["SLE-NM"])
    calc.addForce(N=-100*KN, M= 145*KNm, T=120*KN, descr="force_3",
                  limit_state="serviceability", frequency="quasi-permanent",
                  check_required=["SLE-NM", "SLE-F"])
    calc.addForce(N=-125*KN, M= 125*KNm, T=125*KN, descr="force_4",
                  limit_state="serviceability", frequency="frequent",
                  check_required=["SLE-NM", "SLE-F"])
    calc.addForce(N=-200*KN, M= 200*KNm, T=200*KN, descr="force_5",
                  limit_state="ultimate",
                  check_required=["SLU-T", "SLU-NM"])
    calc.addForce(N=-200*KN, M=-200*KNm, T=100*KN, descr="force_6",
                  limit_state="ultimate",
                  check_required=["SLU-NM"])
    calc.addForce(N=-500*KN, M= 350*KNm, T=200*KN, descr="force_7",
                  limit_state="ultimate",
                  check_required=["SLU-NM"])
    calc.addForce(N= 100*KN, M=-200*KNm, T= 70*KN, descr="force_8",
                  limit_state="ultimate",
                  check_required=["SLU-NM"])

    calc.setMaterialConcrete("NTC2018","C25/30","not aggressive")
    calc.setMaterialRebars("NTC2018", "B450C", "not sensitive")
    calc.addRebarsFromTop(dist_from_top=40, dist_rebars=40, num=4, diam=20)
    calc.addRebarsFromBot(dist_from_bot=40, dist_rebars=40, num=4, diam=20)
    calc.setStirrup(area=100, step=150, angle=90)

    assert calc.run()
    assert calc.buildReport()


